
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "stm32f0xx_conf.h"

#include "ringbuffer.h"
#include "graham-pt.h"

#include "menu.h"
#include "lcd.h"
#include "delay.h"
#include "rtc.h"
#include "date.h"
#include "pcf8563.h"

#include "dbgprintf.h"


typedef int (*fptr)(void);
typedef struct
{
   int  index;
   char *itemName;
   fptr funcptr;
}
struct_menu_t;


// input command ringbuffer
static u32 cmdbuffer[8];
static ringbuffer_t *pRB_Cmd;

static int cmdMode = MODE_MENU_SELECT;

// ProtoThread
static struct pt ptCmdHandler;

static int settime(void);
static int setdate(void);
static PT_THREAD (PT_CmdHandler(struct pt *pt));
static PT_THREAD (PT_TimeSet(struct pt *pt));
static PT_THREAD (PT_DateSet(struct pt *pt));


struct_menu_t menu[] =
{
      { 0, "Set Time", settime },
      { 1, "Set Date", setdate },
      { 255, "", NULL }
};

#define MAX_BACKLIGHT_SETTINGS 5
static uint8_t backlight[MAX_BACKLIGHT_SETTINGS] =
{
      50, 100, 150, 200, 255
};
static uint8_t backlightidx = 0;

extern struct rtc_time rtcTime;


int menu_GetCmdMode(void)
{
   return cmdMode;
}

void menu_init(void)
{
   pRB_Cmd = (ringbuffer_t *)cmdbuffer;
   ringbuffer_init(pRB_Cmd, sizeof(cmdbuffer));

   PT_INIT(&ptCmdHandler);
}

void menu_loop(void)
{
   PT_CmdHandler(&ptCmdHandler);
}

void menu_keymap(uint8_t ir[4])
{
   static uint32_t prev_tick = 0;
   uint8_t key = KEY_NOP;
   uint8_t accept = 0;

   // check manufacturer's code, and also 3rd byte is complement of 4th byte.
   if ((ir[0] == IR_MANU_1) && (ir[1] == IR_MANU_2) && (!(ir[2] & ir[3])))
   {
      switch(ir[2])
      {
      case IR_MUTE:
         key = KEY_MENU;
         break;
      case IR_POWER:
         key = KEY_SELECT;
         break;
      case IR_VOLPLUS:
         key = KEY_UP;
         break;
      case IR_VOLMINUS:
         key = KEY_DOWN;
         break;
      case IR_EFFECT:
         key = KEY_RETURN;
         break;
      case IR_SRCMIX:
         key = KEY_RIGHT;
         break;
      case IR_SRCDIGITAL:
         key = KEY_LEFT;
         break;
      case IR_SRCANALOG:
         key = KEY_BACKLIGHT;
         break;
      default:
         key = KEY_NOP;
         break;
      }

      if (IsDelayOver(prev_tick, 200) > 0) // discard any keys that arrives within 200ms from the last one.
      {
         accept = 1;
      }
   }
   else if ((ir[0] == IR1_MANU_1) && (ir[1] == IR1_MANU_2) && (!(ir[2] & ir[3]))) // MP3 remote control
   {
      switch(ir[2])
      {
      case IR1_MODE:
         key = KEY_MENU;
         break;
      case IR1_PLAYPAUSE:
         key = KEY_SELECT;
         break;
      case IR1_PLUS:
         key = KEY_UP;
         break;
      case IR1_MINUS:
         key = KEY_DOWN;
         break;
      case IR1_RTN:
         key = KEY_RETURN;
         break;
      case IR1_SKIPFWD:
         key = KEY_RIGHT;
         break;
      case IR1_SKIPBACK:
         key = KEY_LEFT;
         break;
      case IR1_EQ:
         key = KEY_BACKLIGHT;
         break;
      default:
         key = KEY_NOP;
         break;
      }

      if (IsDelayOver(prev_tick, 200) > 0) // discard any keys that arrives within 200ms from the last one.
      {
         accept = 1;
      }
   }

   if (accept)
   {
      prev_tick = syscnt;
      if (ringbuffer_available(pRB_Cmd))
      {
         ringbuffer_write(pRB_Cmd, &key, 1);
      }
   }

}


static PT_THREAD (PT_CmdHandler(struct pt *pt))
{
   //PT_YIELDING();
   static uint32_t start_tick = 0;
   static int cursor = 0;
   static struct pt pt_ts;
   static struct pt pt_ds;
   char inchar;
   int rd;
   int i;

   PT_BEGIN(pt);

   while(1)
   {
      if (cmdMode == MODE_MENU_IDLE)
      {
         PT_WAIT_UNTIL(pt, !ringbuffer_isempty(pRB_Cmd));
         rd = ringbuffer_read(pRB_Cmd, (u8 *)&inchar, sizeof(inchar));
         if (inchar == KEY_MENU)
         {
            cursor = 0;
            cmdMode = MODE_MENU_SELECT;
         }
         else if (inchar == KEY_BACKLIGHT)
         {
            LCD_PWMBacklight(backlight[backlightidx]);
            backlightidx++;
            backlightidx %= MAX_BACKLIGHT_SETTINGS;
         }
      }
      else if (cmdMode == MODE_MENU_SELECT)
      {
         i = 0;
         while(menu[i].index != 255)
         {
            LCD_setCursor(2, i);
            LCD_PrintStr((uint8_t *)(menu[i].itemName));
            i++;
         }

         if (cursor == 0)
         {
            LCD_setCursor(0, 0);
            LCD_PrintStr((uint8_t *)"* ");
            LCD_setCursor(0, 1);
            LCD_PrintStr((uint8_t *)"  ");
         }
         else if (cursor == 1)
         {
            LCD_setCursor(0, 0);
            LCD_PrintStr((uint8_t *)"  ");
            LCD_setCursor(0, 1);
            LCD_PrintStr((uint8_t *)"* ");
         }

         start_tick = syscnt;
         PT_WAIT_UNTIL(pt, (!ringbuffer_isempty(pRB_Cmd) || (IsDelayOver(start_tick, 5000) > 0)));
         if (ringbuffer_isempty(pRB_Cmd))
         {
            // That means 5000 msec has elapsed, so we quit time setting menu..
            LCD_ClearDisplay();
            cmdMode = MODE_MENU_IDLE;
         }
         else
         {
            rd = ringbuffer_read(pRB_Cmd, (u8 *)&inchar, sizeof(inchar));
            switch(inchar)
            {
            // Note: Cannot use protothread macros within this switch.
            case KEY_UP:
               if (cursor) cursor--;
               break;
            case KEY_DOWN:
               if (menu[cursor+1].index != 255)
                  cursor++;
               break;
            case KEY_SELECT:
               menu[cursor].funcptr();
               break;
            case KEY_RETURN:
               LCD_ClearDisplay();
               cmdMode = MODE_MENU_IDLE;
               break;
            default:
               break;
            }
         }
      }
      else if (cmdMode == MODE_TIME_SET)
      {
         PT_SPAWN(pt, &pt_ts, PT_TimeSet(&pt_ts));
         cmdMode = MODE_MENU_SELECT;
      }
      else if (cmdMode == MODE_DATE_SET)
      {
         PT_SPAWN(pt, &pt_ds, PT_DateSet(&pt_ds));
         cmdMode = MODE_MENU_SELECT;
      }

   }

   PT_END(pt);
}


static int settime(void)
{
   cmdMode = MODE_TIME_SET;
   return 0;
}


static int setdate(void)
{
   cmdMode = MODE_DATE_SET;
   return 0;
}


static PT_THREAD (PT_TimeSet(struct pt *pt))
{
   //PT_YIELDING();
   static uint8_t run = 1;
   static uint8_t timeidx = 0;
   // timeval[0] is ten hours, timeval[1] is unit hours, timeval[2] is ten minutes, timeval[3] is unit minutes, ..
   static uint8_t timeval[6] = {0,};
   const uint8_t colmap[6] = {0, 1, 3, 4, 6, 7}; // This maps timeval's index to LCD col position for xx:xx:xx
   char inchar;
   char strbuf[17];
   int rd;

   PT_BEGIN(pt);

   run = 1;
   timeidx = 0;
   LCD_ClearDisplay();
   LCD_setCursor(0, 0);
   LCD_PrintStr((uint8_t *)"Set Time");
   LCD_setCursor(0, 1);
   LCD_Blink();

   // get the current time into timeval[]
   timeval[0] = rtcTime.tm_hour / 10;  // get the tens hour
   timeval[1] = rtcTime.tm_hour % 10;  // get the unit hour
   timeval[2] = rtcTime.tm_min / 10;
   timeval[3] = rtcTime.tm_min % 10;
   timeval[4] = rtcTime.tm_sec / 10;
   timeval[5] = rtcTime.tm_sec % 10;

   while(run)
   {
      LCD_setCursor(0, 1);
      sniprintf(strbuf, sizeof(strbuf), "%d%d:%d%d:%d%d",
            timeval[0], timeval[1], timeval[2], timeval[3], timeval[4], timeval[5]);
      LCD_PrintStr((uint8_t *)strbuf);
      LCD_setCursor(colmap[timeidx], 1);

      PT_WAIT_UNTIL(pt, !ringbuffer_isempty(pRB_Cmd));
      rd = ringbuffer_read(pRB_Cmd, (u8 *)&inchar, sizeof(inchar));
      switch(inchar)
      {
      // Note: Cannot use protothread macros within this switch.
         case KEY_UP:
         {
            switch(timeidx)
            {
            case 0: // ten hours
               timeval[timeidx]++;
               if (timeval[timeidx] >= 2)
               {
                  if (timeval[1] > 3) // if unit hours is more than 3, then limit ten hours to 1.
                  {
                     timeval[timeidx] = 1;
                  }
                  else
                  {
                     timeval[timeidx] = 2;
                  }
               }
               break;
            case 1: // unit hours
               timeval[timeidx]++;
               if (timeval[0] == 2 && timeval[timeidx] > 3)
               {
                  timeval[timeidx] = 3;
               }
               else if (timeval[timeidx] > 9)
               {
                  timeval[timeidx] = 9;
               }
               break;
            case 2: // ten minutes
            case 4: // ten seconds
               timeval[timeidx]++;
               if (timeval[timeidx] >= 5)
               {
                  timeval[timeidx] = 5;
               }
               break;
            case 3: // unit minutes
            case 5: // unit seconds
               timeval[timeidx]++;
               if (timeval[timeidx] > 9)
               {
                  timeval[timeidx] = 9;
               }
               break;
            default:
               break;
            }
         }
         break;
         case KEY_DOWN:
         {
            switch(timeidx)
            {
            case 0: // tenth hours
            case 1: // unit hours
            case 2: // ten minutes
            case 3: // unit minutes
            case 4: // ten seconds
            case 5: // unit seconds
               if (timeval[timeidx] > 0)
               {
                  timeval[timeidx]--;
               }
               break;
            default:
               break;
            }
         }
         break;
         case KEY_RIGHT:
         {
            timeidx++;
            timeidx %= (sizeof(timeval)/sizeof(uint8_t));
         }
         break;
         case KEY_LEFT:
         {
            if (timeidx)
               timeidx--;
            else
               timeidx = (sizeof(timeval)/sizeof(uint8_t)) - 1;
         }
         break;
         case KEY_SELECT:
         {
            struct rtc_time tim;
            sniprintf(strbuf, sizeof(strbuf), "%d%d:%d%d:%d%d",
                  timeval[0], timeval[1], timeval[2], timeval[3], timeval[4], timeval[5]);
            parse_time(strbuf, &tim);
            pcf8563_settime(&tim);
            run = 0;
         }
         break;
         case KEY_RETURN:
         {
            run = 0;
         }
         break;
         default:
         break;
      }
   }

   LCD_NoBlink();
   PT_END(pt);
}


static PT_THREAD (PT_DateSet(struct pt *pt))
{
   static uint8_t run = 1;
   static uint8_t dateidx = 0;
   static uint8_t dateval[8] = {0,};
   const uint8_t colmap[8] = {0, 1, 3, 4, 6, 7, 8, 9}; // This maps dateval index to LCD col position for dd-mm-yyyy
   char inchar;
   char strbuf[17];
   int rd;
   int year;

   PT_BEGIN(pt);

   run = 1;
   dateidx = 0;
   LCD_ClearDisplay();
   LCD_setCursor(0, 0);
   LCD_PrintStr((uint8_t *)"Set Date");
   LCD_setCursor(0, 1);
   LCD_Blink();

   dateval[0] = rtcTime.tm_mday / 10;
   dateval[1] = rtcTime.tm_mday % 10;
   dateval[2] = (rtcTime.tm_mon + 1) / 10;
   dateval[3] = (rtcTime.tm_mon + 1) % 10;

   year = (YEAR_BASE + rtcTime.tm_year);
   dateval[4] = year / 1000;
   year %= 1000;
   dateval[5] = year / 100;
   year %= 100;
   dateval[6] = year / 10;
   year %= 10;
   dateval[7] = year;

   while(run)
   {
      LCD_setCursor(0, 1);
      sniprintf(strbuf, sizeof(strbuf), "%d%d-%d%d-%d%d%d%d",
            dateval[0], dateval[1], dateval[2], dateval[3], dateval[4], dateval[5], dateval[6], dateval[7]);
      LCD_PrintStr((uint8_t *)strbuf);
      LCD_setCursor(colmap[dateidx], 1);

      PT_WAIT_UNTIL(pt, !ringbuffer_isempty(pRB_Cmd));
      rd = ringbuffer_read(pRB_Cmd, (u8 *)&inchar, sizeof(inchar));
      switch(inchar)
      {
      // Note: Cannot use protothread macros within this switch.
         case KEY_UP:
         {
            switch(dateidx)
            {
            case 0: // ten days
               dateval[dateidx]++;
               if (dateval[dateidx] >= 3)
               {
                  if (dateval[1] > 1) // if unit days is more than 3, then limit ten days to 1.
                  {
                     dateval[dateidx] = 3;
                  }
                  else
                  {
                     dateval[dateidx] = 2;
                  }
               }
               break;
            case 1: // unit days
               dateval[dateidx]++;
               if (dateval[0] == 3 && dateval[dateidx] > 1)
               {
                  dateval[dateidx] = 1;
               }
               else if (dateval[dateidx] > 9)
               {
                  dateval[dateidx] = 9;
               }
               break;
            case 2: // ten months
               dateval[dateidx]++;
               if (dateval[dateidx] >= 1)
               {
                  dateval[dateidx] = 1;
               }
               break;
            case 3: // unit months
               dateval[dateidx]++;
               if (dateval[2] == 1 && dateval[dateidx] > 2)
               {
                  dateval[dateidx] = 2;
               }
               else if (dateval[dateidx] > 9)
               {
                  dateval[dateidx] = 9;
               }
               break;
            case 4: // thousand years
               dateval[dateidx]++;
               if (dateval[dateidx] > 2) // not going to year 3000..
               {
                  dateval[dateidx] = 2;
               }
               break;
            case 5: // hundred years
               dateval[dateidx]++;
               if (dateval[dateidx] > 9)
               {
                  dateval[dateidx] = 9;
               }
               break;
            case 6: // ten years
               dateval[dateidx]++;
               if (dateval[dateidx] > 9)
               {
                  dateval[dateidx] = 9;
               }
               break;
            case 7: // unit years
               dateval[dateidx]++;
               if (dateval[dateidx] > 9)
               {
                  dateval[dateidx] = 9;
               }
               break;
            default:
               break;
            }
         }
         break;
         case KEY_DOWN:
         {
            switch(dateidx)
            {
            case 0: // ten days
            case 2: // ten months
               if (dateval[dateidx] > 0)
               {
                  dateval[dateidx]--;
               }
               break;
            case 1: // unit days
            case 3: // unit months
               if (dateval[dateidx] > 1) // minimum for unit day/month is 1
               {
                  dateval[dateidx]--;
               }
               break;
            case 4: // thousand years
               if (dateval[dateidx] < 1)
               {
                  dateval[dateidx] =  1;
               }
               break;
            case 5: // hundred years
               if (dateval[dateidx] > 0)
               {
                  dateval[dateidx]--;
               }
               break;
            case 6: // ten years
               if (dateval[dateidx] > 0)
               {
                  dateval[dateidx]--;
               }
               break;
            case 7: // unit years
               if (dateval[dateidx] > 0)
               {
                  dateval[dateidx]--;
               }
               break;
            default:
               break;
            }
         }
         break;
         case KEY_RIGHT:
         {
            dateidx++;
            dateidx %= (sizeof(dateval)/sizeof(uint8_t));
         }
         break;
         case KEY_LEFT:
         {
            if (dateidx)
               dateidx--;
            else
               dateidx = (sizeof(dateval)/sizeof(uint8_t)) - 1;
         }
         break;
         case KEY_SELECT:
         {
            struct rtc_time tim;
            sniprintf(strbuf, sizeof(strbuf), "%d%d-%d%d-%d%d%d%d",
                  dateval[0], dateval[1], dateval[2], dateval[3], dateval[4], dateval[5], dateval[6], dateval[7]);
            parse_date(strbuf, &tim);
            pcf8563_setdate(&tim);
            run = 0;
         }
         break;
         case KEY_RETURN:
         {
            run = 0;
         }
         break;
         default:
         break;
      }
   }

   LCD_NoBlink();
   PT_END(pt);
}








